/***********************************************************************
 * File: byteorder.h
 * 	(Extracted from
 *	   "$Id: byteorder.h,v 1.4 2007/10/21 06:16:26 Administrator Exp Administrator $")
 *
 * Abstracts:
 *  This C header file contains definitions of `byteorder' API-s.
 *
 * Copyright (C) 2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _BYTEORDER_H_ /* { */
#define _BYTEORDER_H_

/* buffer_editor needs the bellow system include files */
#include <unistd.h>
#include <limits.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>
#include <errno.h>

/* definitions of converting directions */
typedef enum byteorder_dir_e {
	  Byteorder_dir_e_ntoh	/* network-byteorder to host's one */
	, Byteorder_dir_e_hton	/* host-byteorder to network's one */
	, Byteorder_dir_e_N	/* number of converting directions */
} Byteorder_dir_e;

/* type definition of the field layout network and host buffers */
typedef struct byteorder_layout_t {
	enum byteorder_type_e {
		  Byteorder_type_e_char		/* unsigned 8bits */
		, Byteorder_type_e_short	/* unsigned 16bits */
		, Byteorder_type_e_long		/* unsigned 32bits */
		, Byteorder_type_e_N		/* number of types */
		}	type;		 /* type of a field */
	size_t		occurrence;	 /* element number of array */
	size_t		net_fld_offset;	 /* offset of network buffer */
	size_t		host_fld_offset; /* offset of host buffer */
} Byteorder_layout_t;

/* Macro: get size of the typed field */
#define Byteorder_type_to_size(TYPE) (				\
		((TYPE) == Byteorder_type_e_char)?		\
						sizeof(char)	\
		: ((TYPE) == Byteorder_type_e_short)?		\
						sizeof(short)	\
		: ((TYPE) == Byteorder_type_e_long)?		\
						sizeof(long)	\
		:				(size_t)0	\
		)

/* Byteorder converting for the sequence of 32bits integers
 * Arguments;
 *	o `direction' indicates converting directions;
 *		(1) Byteorder_dir_e_ntoh: network-wide to host-native
 *		(2) Byteorder_dir_e_hton: host-native to network-wide
 *	o `netp' points to the sequence of network-byteorder 32bits
 *	 integers.
 *	o `hostp' points to the sequence of host-byteorder 32bits
 *	 integers.
 *	o `occurrence' indicates the number of integers sequences.
 * Returns;
 *	NULL    : several errors occurred.
 *	non-NULL: return normally
 */
extern char *byteorder_longs(Byteorder_dir_e	direction
			, void			*netp
			, void			*hostp
			, size_t		occurrence
			);

/* Byteorder converting for the sequence of 16bits integers
 * Arguments;
 *	o `direction' indicates converting directions;
 *		(1) Byteorder_dir_e_ntoh: network-wide to host-native
 *		(2) Byteorder_dir_e_hton: host-native to network-wide
 *	o `netp' points to the sequence of network-byteorder 16bits
 *	 integers.
 *	o `hostp' points to the sequence of host-byteorder 16bits
 *	 integers.
 *	o `occurrence' indicates the number of integers sequences.
 * Returns;
 *	NULL    : several errors occurred.
 *	non-NULL: return normally
 */
extern char *byteorder_shorts(Byteorder_dir_e	direction
			, void			*netp
			, void			*hostp
			, size_t		occurrence
			);

/* Byteorder converting for the sequence of 8bits integers
 * Arguments;
 *	o `direction' indicates converting directions;
 *		(1) Byteorder_dir_e_ntoh: network-wide to host-native
 *		(2) Byteorder_dir_e_hton: host-native to network-wide
 *	o `netp' points to the sequence of network-byteorder 8bits
 *	 integers.
 *	o `hostp' points to the sequence of host-byteorder 8bits
 *	 integers.
 *	o `occurrence' indicates the number of integers sequences.
 * Returns;
 *	NULL    : several errors occurred.
 *	non-NULL: return normally
 */
extern char *byteorder_chars(Byteorder_dir_e	direction
			, char			*netp
			, char			*hostp
			, size_t		occurrence
			);

/* Byteorder converting for the differ buffers.
 * Arguments;
 *	o `direction' indicates converting directions;
 *		(1) Byteorder_dir_e_ntoh: network-wide to host-native
 *		(2) Byteorder_dir_e_hton: host-native to network-wide
 *	o `net' points to the sequence of network-byteorder 16bits
 *	 integers.
 *	o `net_rec_sz' indicates the size of `net' records.
 *	o `layouts_n' indicates the number of elements of `layouts'
 *	 array.
 *	o `layouts' points to the array of layout of the converting
 *	 fields.
 *	o `hostp' points to the sequence of host-byteorder 16bits
 *	 integers.
 *	o `host_rec_sz' indicates the size of `host' records.
 * Returns;
 *	NULL    : several errors occurred.
 *	non-NULL: return normally
 */
extern char *byteorder(Byteorder_dir_e		direction
			, void			*netp
			, size_t		net_rec_sz
			, size_t		layouts_n
			, Byteorder_layout_t	*layouts
			, void			*hostp
			, size_t		host_rec_sz
			);

#endif /* } !defined(_BYTEORDER_H_) */
